require 'securerandom'

# Add ID and timestamp attribute to Exception
# and modify its constructor and inspect method.
# Also add methods to_hash and to_json to allow
# transparent JSON serialization of all errors.

# @private
class Exception # :nodoc:

  # Unique exception instance ID.
  # @return [String] Unique ID.
  attr_reader :id

  # Time of exception instantiation.
  # @return [Time] Time of exception instantiation.
  attr_reader :timestamp

  # Alias constructor so we can extend it.
  alias_method :_initialize, :initialize
  private :_initialize

  # Creates new instance of Exception.
  # @private
  def initialize(*args) # :nodoc:
    _initialize(*args)
    @id = SecureRandom.uuid() if @id.nil?
    @timestamp = Time.now() if @timestamp.nil?
  end

  # Override default inspect to inject ID.
  # @return [String] String representation.
  # @private
  def inspect # :nodoc:
    "#<#{self.class.name} [#{id}]: #{message}>"
  end

  # Serializes exception to JSON.
  # @private
  def to_json(*args) # :nodoc:
    return to_hash.to_json(*args)
  end

  protected

  # Prepares structure with basic error data
  # to be modified/extended by ancestors. This
  # structure is then serialized to JSON.
  # @private
  def to_hash # :nodoc:
    hash = {id: id, timestamp: timestamp}
    hash[:message] = message
    return hash
  end

end
