require 'time'

module UU
  module OS
    module Lang

      # Module providing extension to {Hash} to be able to easily convert stored
      # values to given type. By default only basic data types are supported (any
      # type to be supported must implement static method +__convert(value)+).
      # @private 
      module HashConverter # :nodoc:

        # Returns a value from the hash for the given key
        # transformed to instance of given type (returned
        # object may not be exact instance of given type,
        # however should be instance of object with same
        # behavior).
        # @param [Object] key Key
        # @param [Class] type Desired value type
        # @return [Object] Object of given type
        # @raise [TypeError] In case transformation is not possible
        def get(key, type = nil)
          val = self[key]
          return (val.nil? || type.nil? || val.kind_of?(type)) ? val : type.__convert(val)
        end

      end

    end
  end
end

# @private
class ::Object # :nodoc:
  # By default no conversion is supported.
  def self.__convert(value)
    raise TypeError, "no conversion of given #{value.class} into #{self}"
  end
end

# @private
class ::String # :nodoc:
  def self.__convert(value)
    case value
    when String
      return value
    else
      return value.to_s
    end
  end
end

# @private
class ::TrueClass # :nodoc:
  def self.__convert(value)
    case value
    when TrueClass
      return true
    when FalseClass
      return false
    when String && /^true$/i
      return true
    when String && /^false$/i
      return false
    when Numeric && 1
      return true
    when Numeric && 0
      return false
    else
      super(value)
    end
  end
end

# @private
class ::FalseClass # :nodoc:
  def self.__convert(value)
    case value
    when TrueClass
      return true
    when FalseClass
      return false
    when String && /^true$/i
      return true
    when String && /^false$/i
      return false
    when Numeric && 1
      return true
    when Numeric && 0
      return false
    else
      super(value)
    end
  end
end

# @private
class ::Numeric # :nodoc:
  def self.__convert(value)
    case value
    when Numeric
      return value
    when String && /^[+-]?[\d]+$/
      return value.to_i
    else
      super(value)
    end
  end
end

# @private
class ::Float # :nodoc:
  def self.__convert(value)
    case value
    when Numeric
      return value
    when String && /^[+-]?[\d]+(\.[\d]+)?([eE][+-]?[\d]+)?$/
      return value.to_f
    else
      super(value)
    end
  end
end

# @private
class ::Date # :nodoc:
  def self.__convert(value)
    case value
    when DateTime, Time
      return Date.parse(value.to_s)
    when Date
      return value
    when String
      return Date.parse(value) rescue super(value)
    else
      super(value)
    end
  end
end

# @private
class ::Time # :nodoc:
  def self.__convert(value)
    case value
    when Date, DateTime
      return Time.parse(value.to_s)
    when Time
      return value
    when String
      return Time.parse(value) rescue super(value)
    else
      super(value)
    end
  end
end

# @private
class ::DateTime # :nodoc:
  def self.__convert(value)
    case value
    when DateTime
      return value
    when Date, Time
      return DateTime.parse(value.to_s)
    when String
      return DateTime.parse(value) rescue super(value)
    else
      super(value)
    end
  end
end
